@@ -27,7 +27,7 @@ use Carp;
 
 use 5.10.0;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Params::Check qw[ check ];
 use Ref::Util ':all';
@@ -238,14 +238,14 @@ sub _reset_effective_data_storage {
 
     if ( $self->apply_mask && $is_subscribed ) {
 
-	$self->_set_PDL( $self->base->copy )
-	  if $self->_has_shared_data_storage;
+        $self->_set_PDL( $self->base->copy )
+          if $self->_has_shared_data_storage;
     }
 
     # otherwise, save some space.
     else {
 
-	$self->_set_PDL( $self->base )
+        $self->_set_PDL( $self->base );
     }
 
     return;
@@ -254,10 +254,31 @@ sub _reset_effective_data_storage {
 sub is_subscribed {
 
     return $_[0]->has_mask && $_[0]->_has_token;
-    return $_[0]->has_mask;
 
 }
 
+sub _reset_subscription_status {
+
+    $_[0]->_clear_token;
+
+}
+
+sub _reset_subscription_state {
+
+    my $self = shift;
+    my $reset_data_storage = shift // 1;
+
+    $self->_reset_subscription_status;
+
+    $self->_reset_effective_data_storage
+      if $reset_data_storage;
+
+    $self->update;
+
+    return;
+}
+
+
 sub subscribe {
 
     my $self = shift;
@@ -268,15 +289,38 @@ sub subscribe {
     # $self->base
     $self->_reset_effective_data_storage( 1 );
 
-    my $token = $self->mask->subscribe(
-        (
+    my $token = $self->mask->subscribe( (
             $self->apply_mask
-            ? ( apply_mask => sub { $self->_apply_mask( @_ ) } )
+            ? (
+                apply_mask => sub {
+
+                    return unless $self->is_subscribed;
+
+                    # if $mask is undef, we're being asked to unsubscribe
+                    if ( @_ == 0 || !defined $_[0] ) {
+
+                        # if mask has already unsubscribed us; don't
+                        # ask it to do it twice
+                        if ( $self->mask->is_subscriber( $self->_token ) ) {
+                            $self->unsubscribe;
+                        }
+
+                        # mask doesn't know about us. just clean up locally
+                        else {
+                            $self->_reset_subscription_state;
+                        }
+                    }
+                    else {
+                        $self->_apply_mask( @_ );
+                    }
+
+                    return;
+                },
+              )
             : (),
         ),
         (
-            $self->data_mask
-	    ? ( data_mask => sub { $self->_data_mask } )
+            $self->data_mask ? ( data_mask => sub { $self->_data_mask } )
             : ()
         ),
         ( $self->_has_token ? ( token => $self->_token ) : () ),
@@ -295,25 +339,28 @@ sub unsubscribe {
 
     return unless $self->is_subscribed;
 
-    state $tmpl = {
-		   reset_data_storage => { default => 1, strict_type => 1, defined => 1 }
-		  };
+    state $tmpl
+      = {
+        reset_data_storage => { default => 1, strict_type => 1, defined => 1 }
+      };
 
     my $opts = check( $tmpl, {@_} )
       or die Params::Check::last_error();
 
-    $self->mask->unsubscribe( $self->_token );
-    $self->_clear_token;
+    # make sure $self doesn't think it's subscribed, in case
+    # $self->mask sends *us* an unsubscribe command via apply_mask
+    # when we call mask->unsuscribe method
+    my $token = $self->_token;
+    $self->_reset_subscription_status;
 
-    $self->_reset_effective_data_storage
-      if $opts->{reset_data_storage};
+    $self->mask->unsubscribe( $token );
 
-    $self->update;
+    # now perform a reset of all subscription related state
+    $self->_reset_subscription_state( $opts->{reset_data_storage} );
 
     return;
 }
 
-
 sub _apply_mask {
 
     my $self = shift;
@@ -352,24 +399,24 @@ sub update {
     # a mask is involved
     if ( $self->is_subscribed ) {
 
-	if ( $self->data_mask ) {
+        if ( $self->data_mask ) {
 
-	    # this will automatically apply the mask if required
-	    $self->mask->update;
-	}
+            # this will automatically apply the mask if required
+            $self->mask->update;
+        }
 
-	elsif ( $self->apply_mask ) {
+        elsif ( $self->apply_mask ) {
 
-	    $self->_apply_mask( $self->mask );
-	}
+            $self->_apply_mask( $self->mask );
+        }
 
     }
 
     # no mask, but PDL & base don't share storage
 
-    elsif ( ! $self->_has_shared_data_storage ) {
+    elsif ( !$self->_has_shared_data_storage ) {
 
-	$self->{PDL} .= $self->base;
+        $self->{PDL} .= $self->base;
     }
 
     return;
